/*
 * The default style sheet used by khtml to render HTML pages
 * Copyright 2000-2003 Lars Knoll (knoll@kde.org)
 *
 * Konqueror/khtml relies on the existence of this style sheet for
 * rendering. Do not remove or modify this file unless you know
 * what you are doing.
 *
 * KHTML_STYLE_VERSION: 1
 */

@namespace "http://www.w3.org/1999/xhtml";

html {
	display: block;
	color: -khtml-text;
}

/*
 * head and it's children all have display=none
 */

head {
	display: none;
}

meta {
	display: none;
}

title {
	display: none;
}

link {
	display: none;
}

style {
	display: none;
}

script {
	display: none;
}

/*
 * generic block level elements
 */

@media print {
	body {
		display: block;
		margin: 0px;
	}
}

@media screen {
	body {
		display: block;
		margin: 10px;
	}
}

p {
	display: block;
	margin: 1.0__qem 0px;
}

div {
	display: block;
}

/* to force a block level context for some cases (broken HTML) */

layer {
	display: block;
}

address {
	display: block;
}

blockquote {
	display: block;
        margin: 1__qem 40px 1em 40px;
}

/* default behaviour:
:lang(en)  > q { quotes: '"' '"' "'" "'" }
*/

:lang(ar) > q, :lang(fa) > q { quotes: "\00AB" "\00BB"; }
:lang(ca) > q, :lang(es) > q { quotes: "\00AB" "\00BB" "\0022" "\0022"; }
:lang(el) > q, :lang(pt) > q, :lang(tr) > q, :lang(vi) > q { quotes: "\00AB" "\00BB" "\2039" "\203A"; }
:lang(hy) > q { quotes: "\00AB" "\00BB" "\201C" "\201D"; }
:lang(ru) > q, :lang(uk) > q { quotes: "\00AB" "\00BB" "\201E" "\201C"; }
:lang(eo) > q { quotes: "\201C" "\201D"; }
:lang(id) > q, :lang(it) > q, :lang(zh-cn) > q, :lang(ko) > q, :lang(nb) > q, :lang(no) > q { quotes: "\201C"  "\201D"  "\2018" "\2019"; }
:lang(bg) > q { quotes: "\201E" "\201C"; }
:lang(cs) > q, :lang(de) > q, :lang(lt) > q, :lang(sk) > q, :lang(sl) > q { quotes: "\201E" "\201C" "\201A" "\2018"; }
:lang(da) > q { quotes: "\201E" "\201C" "\2019" "\2019"; }
:lang(pl) > q, :lang(ro) > q { quotes: "\201E" "\201D" "\00AB" "\00BB"; }
:lang(hu) > q { quotes: "\201E" "\201D" "\00BB"  "\00AB"; }
:lang(fi) > q, :lang(nl) > q, :lang(sv) > q { quotes: "\201D" "\201D"  "\2019" "\2019"; }
:lang(fr) > q { quotes: "\00AB\00A0" "\00A0\00BB" "\2039\00A0" "\00A0\203A"; }
:lang(hr) > q { quotes: "\00BB" "\00AB" "\203A" "\2039"; }
:lang(ja) > q, :lang(zh-hk) > q, :lang(zh-tw) > q { quotes: "\300C" "\300D" "\300E" "\300F"; }

q        {
         display: inline;
}

q:before {
        content: open-quote;
}

q:after  {
        content: close-quote;
}

center {
	display: block;
	/* special centering to be able to emulate the html4/netscape behavior */
	text-align: -khtml-center;
}

hr  {
        display: block;
        margin: 12px auto;
        border-style: inset;
        border-width: 1px;
        -khtml-flow-mode: -khtml-around-floats
}

map {
	display: inline;
}

/*
 * heading elements
 */

h1 {
        display: block;
        margin: 0.67__qem 0 0.67em 0;
        font-size: 2em;
        font-weight: bold;
}

h2 {
        display: block;
        margin: 0.83__qem 0 0.83em 0;
        font-size: 1.5em;
        font-weight: bold;
}

h3 {
        display: block;
        margin: 1__qem 0 1em 0;
        font-size: 1.17em;
        font-weight: bold;
}

h4 {
        display: block;
        margin: 1.33__qem 0 1.33em 0;
        font-size: 1em;
        font-weight: bold;
}

h5 {
        display: block;
        margin: 1.67__qem 0 1.67em 0;
        font-size: 0.83em;
        font-weight: bold;
}

h6 {
        display: block;
        margin: 2.33__qem 0 2.33em 0;
        font-size: 0.67em;
        font-weight: bold;
}

/*
 * tables
 */

table {
	display: table;
	border-collapse: separate;
	border-spacing: 2px;
        -khtml-flow-mode: -khtml-around-floats;
        box-sizing: border-box;
}

thead {
	display: table-header-group;
        border-color: inherit;
	vertical-align: middle;
}

tbody {
	display: table-row-group;
        border-color: inherit;
	vertical-align: middle;
}

tfoot {
	display: table-footer-group;
        border-color: inherit;
	vertical-align: middle;
}

col {
	display: table-column;
}

colgroup {
	display: table-column-group;
}

tr {
	display: table-row;
	vertical-align: inherit;
        border-color: inherit;
}

td, th {
	display: table-cell;
	vertical-align: inherit;
}

th {
	font-weight: bolder;
}

caption {
	display: table-caption;
	text-align: -khtml-center;
}

/*
 * lists
 */

ul, menu, dir {
        display: block;
        list-style-type: disc;
        margin: 1__qem 0 1em 0;
        -khtml-padding-start: 40px
}

ol {
        display: block;
        list-style-type: decimal;
        margin: 1__qem 0 1em 0;
        -khtml-padding-start: 40px
}

li {
        display: list-item;
}

ul   ul ul,   ul   ol ul,     ul   menu ul,   ul   dir ul,
ol   ul ul,   ol   ol ul,     ol   menu ul,   ol   dir ul,
menu ul ul,   menu ol ul,     menu menu ul,   menu dir ul,
dir  ul ul,   dir  ol ul,     dir  menu ul,   dir  dir ul,
ul   ul menu, ul   ol menu,   ul   menu menu, ul   dir menu,
ol   ul menu, ol   ol menu,   ol   menu menu, ol   dir menu,
menu ul menu, menu ol menu,   menu menu menu, menu dir menu,
dir  ul menu, dir  ol menu,   dir  menu menu, dir  dir menu,
ul   ul dir,  ul   ol dir,    ul   menu dir,  ul   dir dir,
ol   ul dir,  ol   ol dir,    ol   menu dir,  ol   dir dir,
menu ul dir,  menu ol dir,    menu menu dir,  menu dir dir,
dir  ul dir,  dir  ol dir,    dir  menu dir,  dir  dir dir
{
  list-style-type: square;
}

ul   ul, ul   menu, ul   dir,
ol   ul, ol   menu, ol   dir,
menu ul, menu menu, menu dir,
dir  ul, dir  menu, dir  dir
{
  list-style-type: circle;
}

dd {
	display: block;
}

dl > dd {
	-khtml-margin-start: 40px;
}

dl {
	display: block;
        margin: 1__qem 0 1em 0;
}

dt {
	display: block;
}

ol   ul, ol   menu, ol   dir, ol   ol,
ul   ul, ul   menu, ul   dir, ul   ol,
menu ul, menu menu, menu dir, menu ol,
dir  ul, dir  menu, dir  dir, dir  ol
{
	margin-top: auto;
	margin-bottom: auto;
}

li > p {
	margin-top: auto;
/*	margin-bottom: auto;*/
}

li > div {
	margin-top: auto;
/*	margin-bottom: auto;*/
}

/*
 * form elements
 */

form {
	display: block;
        margin: 0__qem 0 1em 0;
}

legend {
        display: block;
        padding-left: 2px;
        padding-right: 2px;
        border: none;
        margin: 0;
}

fieldset {
	display: block;
        padding: 0.75em 0.625em;
	margin: 1.0em 0;
        border: 2px groove threedface;
        -khtml-flow-mode: -khtml-around-floats
}

button {
        display: inline-block;
        border: 2px outset buttonface;
        background-color: buttonface;
        font-size: small;
        color: buttontext;
        padding: 2px 2px 2px 2px;
        cursor: default;
        box-sizing: border-box;
}

button:enabled:active {
        border-style: inset;
}

input, textarea {
        text-align: -khtml-auto;
}

input, textarea, select, button {
        font-weight: normal;
        margin: 0__qem;
}
input:focus, textarea:focus, select:focus, button:focus {
        outline: none;
}

input { color: windowtext;
        font-family: sans-serif;
        font-size: small;
        border: 2px -khtml-native;
        background-color: window;
        cursor: text;
}

input[type="hidden"] {
	display: none;
}

input[type="radio"], input[type="checkbox"] {
        margin: 0 0.5ex;
        background-color: window;
        color: windowtext;
        box-sizing: border-box;
        cursor: default;
}

input[type="text"], input[type="password"] {
        background-color: window;
        color: windowtext;
	cursor: text;
}

input[type="submit"], input[type="reset"], input[type="button"] {
        background-color: buttonface;
        color: buttontext;
        box-sizing: border-box;
        cursor: default;
}

input[type="file"] {
        background-color: window;
        color: windowtext;
        box-sizing: border-box;
        cursor: default;
}

input[type="image"] {
       background-color: transparent;
       cursor: default;
}

isindex { color: windowtext; font-size: small; border: 2px -khtml-native; }


option,
optgroup,
area,
param  {
	display: none;
}

select {
        font-family: sans-serif;
        font-size: small;
        color: buttontext;
        background-color: window;
        box-sizing: border-box;
        border: 2px -khtml-native;
}

select[multiple],
select[size] {
        color: windowtext;
}

select[size="0"]:not([multiple]),
select[size="1"]:not([multiple]) {
        color: buttontext;
}

textarea {
        color: windowtext;
        background-color: window;
        font-family: monospace;
        border: 2px -khtml-native;
        cursor: text;
        overflow-y: scroll;
}

/*
 * inline elements
 */

u,
ins {
	text-decoration: underline;
}

strong,
b {
	font-weight: bolder;
}

i,
cite,
em,
var,
address {
	font-style: italic;
}

tt,
code,
kbd,
samp {
	font-family: monospace;
}

pre,
listing,
xmp,
plaintext {
	display: block;
	font-family: monospace;
	white-space: pre;
	margin: 1__qem 0;
}

big {
	font-size: larger;
}

small {
	font-size: smaller;
}

s,
strike,
del  {
	text-decoration: line-through;
}

sub {
	vertical-align: sub;
	font-size: smaller;
}
sup {
	vertical-align: super;
	font-size: smaller;
}

abbr, acronym {
	font-variant: small-caps;
	letter-spacing: 0.1em
}

*|:focus          { outline: 1px dotted invert }
a:link:active          { color: red; outline: 1px dotted invert; }
a:visited:active          { color: red; outline: 1px dotted invert; }

/* with the current design it is too expensive to set this default via css
:before,:after  { white-space: pre-line }
*/

/* ### use this to replace renderbr
      br:before       { content: "\n" }
*/


/* bidirectionality settings (do not change) */

bdo[dir="ltr"]  {
	direction: ltr;
	unicode-bidi: bidi-override;
}

bdo[dir="rtl"]  {
	direction: rtl;
	unicode-bidi: bidi-override;
}

      *[dir="ltr"]    { direction: ltr; unicode-bidi: embed }
      *[dir="rtl"]    { direction: rtl; unicode-bidi: embed }

/* elements that are block-level in html4 */
/* ### don't support unicode-bidi at the moment
      address, blockquote, body, dd, div, dl, dt, fieldset,
      form, frame, frameset, h1, h2, h3, h4, h5, h6, iframe,
      noscript, noframes, object, ol, p, ul, applet, center,
      dir, hr, menu, pre, listing, li, table, tr, thead, tbody, tfoot,
      col, colgroup, td, th, caption
                      { unicode-bidi: embed }
*/

/* end bidi settings */

/*
 * other elements
 */

noframes {
	display: none;
}

frameset {
	display: block;
}

frame {
	display: block;
}

iframe:focus, object:focus, frame:focus {
        outline: none;
}

nobr {
        white-space: nowrap;
}

marquee {
    display: inline-block;
    overflow: marquee;
}

/* noscript is handled internally, as it depends on the html settings */

@media print {
  h1, h2, h3,
  h4, h5, h6    { page-break-after: avoid }
  ul, ol, dl    { page-break-before: avoid }
}
