/*
 * The default presentational attributes' styles used by khtml to render HTML pages
 * (cf. CSS 2.1 - 6.4.4)
 * Copyright 2007 Germain Garand (germain@ebooksfrance.org)
 *
 * Konqueror/khtml relies on the existence of this style sheet for
 * rendering. Do not remove or modify this file unless you know
 * what you are doing.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

pre[wrap], listing[wrap] {
    white-space: pre-wrap
}

table[rules="groups"] > tbody, 
table[rules="groups"] > thead, 
table[rules="groups"] > tfoot {
    border-top-width: thin;
    border-bottom-width: thin;
    border-top-style: solid;
    border-bottom-style: solid;
}

table[rules="groups"] > colgroup {
    border-left-width: thin;
    border-right-width: thin;
    border-left-style: solid;
    border-right-style: solid;
}

table[border] > tbody > tr > td,
table[border] > tbody > tr > th,
table[border] > thead > tr > td,
table[border] > thead > tr > th,
table[border] > tfoot > tr > td,
table[border] > tfoot > tr > th,
table[border] > tr > td,
table[border] > tr > th {
    border-style: inset;
    border-width: thin;
}

table[bordercolor] > tbody > tr > td,
table[bordercolor] > tbody > tr > th,
table[bordercolor] > thead > tr > td,
table[bordercolor] > thead > tr > th,
table[bordercolor] > tfoot > tr > td,
table[bordercolor] > tfoot > tr > th,
table[bordercolor] > tr > td,
table[bordercolor] > tr > th {
    border-style: solid;
}

table[border="0"] > tbody > tr > td,
table[border="0"] > tbody > tr > th,
table[border="0"] > thead > tr > td,
table[border="0"] > thead > tr > th,
table[border="0"] > tfoot > tr > td,
table[border="0"] > tfoot > tr > th,
table[border="0"] > tr > td,
table[border="0"] > tr > th {
   /* Do not attempt to avoid this declaration by appending a :not() to the table[border] selectors
    * That would increase its specificity, breaking proper cascade.
    */
    border-style: none;
}

table[rules] > tbody > tr > td,
table[rules] > tbody > tr > th,
table[rules] > thead > tr > td,
table[rules] > thead > tr > th,
table[rules] > tfoot > tr > td,
table[rules] > tfoot > tr > th,
table[rules] > tr > td,
table[rules] > tr > th {
    border-style: none;
}

table[rules="all"] > tbody > tr > td,
table[rules="all"] > tbody > tr > th,
table[rules="all"] > thead > tr > td,
table[rules="all"] > thead > tr > th,
table[rules="all"] > tfoot > tr > td,
table[rules="all"] > tfoot > tr > th,
table[rules="all"] > tr > td,
table[rules="all"] > tr > th {
    border-width: thin;
    border-style: solid;
}

table[rules="cols"] > tbody > tr > td, 
table[rules="cols"] > tbody > tr > th, 
table[rules="cols"] > thead > tr > td, 
table[rules="cols"] > thead > tr > th,
table[rules="cols"] > tfoot > tr > td,
table[rules="cols"] > tfoot > tr > th,
table[rules="cols"] > tr > td,
table[rules="cols"] > tr > th {
    border-left-width: thin;
    border-right-width: thin;
    border-left-style: solid;
    border-right-style: solid;
} 

table[rules="rows"] > tbody > tr > td,
table[rules="rows"] > tbody > tr > th,
table[rules="rows"] > thead > tr > td,
table[rules="rows"] > thead > tr > th,
table[rules="rows"] > tfoot > tr > td,
table[rules="rows"] > tfoot > tr > th,
table[rules="rows"] > tr > td,
table[rules="rows"] > tr > th {
    border-top-width: thin;
    border-bottom-width: thin;
    border-top-style: solid;
    border-bottom-style: solid;
}

table[rules]:not([rules="none"]){
    /* FIXME: enabling border-collapse when 'rules' attribute is defined matches
     * current Mozilla/Opera but is not standard.
     */
    border-collapse: collapse;
}

table[align="center"] {
	margin-left: auto;
	margin-right: auto;
}

table[align="left"] {
	float: -khtml-left;
}

table[align="right"] {
	float: -khtml-right;
}

dl[compact] > dt {
	display: compact;
}
